@@ -5,13 +5,13 @@ from __future__ import annotations
 import html
 import os
 import re
+import time
 from os import path
-from typing import Any, NamedTuple
+from typing import TYPE_CHECKING, Any, NamedTuple
 from urllib.parse import quote
 from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile
 
 from docutils import nodes
-from docutils.nodes import Element, Node
 from docutils.utils import smartquotes
 
 from sphinx import addnodes
@@ -20,8 +20,10 @@ from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.display import status_iterator
 from sphinx.util.fileutil import copy_asset_file
-from sphinx.util.i18n import format_date
-from sphinx.util.osutil import copyfile, ensuredir
+from sphinx.util.osutil import copyfile, ensuredir, relpath
+
+if TYPE_CHECKING:
+    from docutils.nodes import Element, Node
 
 try:
     from PIL import Image
@@ -53,7 +55,7 @@ CSS_LINK_TARGET_CLASS = 'link-target'
 # XXX These strings should be localized according to epub_language
 GUIDE_TITLES = {
     'toc': 'Table of Contents',
-    'cover': 'Cover'
+    'cover': 'Cover',
 }
 
 MEDIA_TYPES = {
@@ -65,9 +67,9 @@ MEDIA_TYPES = {
     '.svg': 'image/svg+xml',
     '.jpg': 'image/jpeg',
     '.jpeg': 'image/jpeg',
-    '.otf': 'application/x-font-otf',
-    '.ttf': 'application/x-font-ttf',
-    '.woff': 'application/font-woff',
+    '.otf': 'font/otf',
+    '.ttf': 'font/ttf',
+    '.woff': 'font/woff',
 }
 
 VECTOR_GRAPHICS_EXTENSIONS = ('.svg',)
@@ -100,8 +102,7 @@ class NavPoint(NamedTuple):
     playorder: int
     text: str
     refuri: str
-    children: list[Any]     # mypy does not support recursive types
-                            # https://github.com/python/mypy/issues/7069
+    children: list[NavPoint]
 
 
 def sphinx_smarty_pants(t: str, language: str = 'en') -> str:
@@ -167,9 +168,9 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.refnodes: list[dict[str, Any]] = []
 
     def create_build_info(self) -> BuildInfo:
-        return BuildInfo(self.config, self.tags, ['html', 'epub'])
+        return BuildInfo(self.config, self.tags, frozenset({'html', 'epub'}))
 
-    def get_theme_config(self) -> tuple[str, dict]:
+    def get_theme_config(self) -> tuple[str, dict[str, str | int | bool]]:
         return self.config.epub_theme, self.config.epub_theme_options
 
     # generic support functions
@@ -183,15 +184,14 @@ class EpubBuilder(StandaloneHTMLBuilder):
         return id
 
     def get_refnodes(
-        self, doctree: Node, result: list[dict[str, Any]]
+        self, doctree: Node, result: list[dict[str, Any]],
     ) -> list[dict[str, Any]]:
         """Collect section titles, their depth in the toc and the refuri."""
         # XXX: is there a better way than checking the attribute
         # toctree-l[1-8] on the parent node?
         if isinstance(doctree, nodes.reference) and doctree.get('refuri'):
             refuri = doctree['refuri']
-            if refuri.startswith('http://') or refuri.startswith('https://') \
-               or refuri.startswith('irc:') or refuri.startswith('mailto:'):
+            if refuri.startswith(('http://', 'https://', 'irc:', 'mailto:')):
                 return result
             classes = doctree.parent.attributes['classes']
             for level in range(8, 0, -1):  # or range(1, 8)?
@@ -199,7 +199,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     result.append({
                         'level': level,
                         'refuri': html.escape(refuri),
-                        'text': ssp(html.escape(doctree.astext()))
+                        'text': ssp(html.escape(doctree.astext())),
                     })
                     break
         elif isinstance(doctree, nodes.Element):
@@ -242,19 +242,19 @@ class EpubBuilder(StandaloneHTMLBuilder):
             'level': 1,
             'refuri': html.escape(self.config.root_doc + self.out_suffix),
             'text': ssp(html.escape(
-                self.env.titles[self.config.root_doc].astext()))
+                self.env.titles[self.config.root_doc].astext())),
         })
         for file, text in reversed(self.config.epub_pre_files):
             refnodes.insert(0, {
                 'level': 1,
                 'refuri': html.escape(file),
-                'text': ssp(html.escape(text))
+                'text': ssp(html.escape(text)),
             })
         for file, text in self.config.epub_post_files:
             refnodes.append({
                 'level': 1,
                 'refuri': html.escape(file),
-                'text': ssp(html.escape(text))
+                'text': ssp(html.escape(text)),
             })
 
     def fix_fragment(self, prefix: str, fragment: str) -> str:
@@ -317,7 +317,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
         def footnote_spot(tree: nodes.document) -> tuple[Element, int]:
             """Find or create a spot to place footnotes.
 
-            The function returns the tuple (parent, index)."""
+            The function returns the tuple (parent, index).
+            """
             # The code uses the following heuristic:
             # a) place them after the last existing footnote
             # b) place them after an (empty) Footnotes rubric
@@ -343,8 +344,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
             nr = 1
         for node in list(tree.findall(nodes.reference)):
             uri = node.get('refuri', '')
-            if (uri.startswith('http:') or uri.startswith('https:') or
-                    uri.startswith('ftp:')) and uri not in node.astext():
+            if uri.startswith(('http:', 'https:', 'ftp:')) and uri not in node.astext():
                 idx = node.parent.index(node) + 1
                 if show_urls == 'inline':
                     uri = self.link_target_template % {'uri': uri}
@@ -418,7 +418,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                                    path.join(self.srcdir, src), err)
                 continue
             if self.config.epub_fix_images:
-                if img.mode in ('P',):
+                if img.mode == 'P':
                     # See the Pillow documentation for Image.convert()
                     # https://pillow.readthedocs.io/en/stable/reference/Image.html#PIL.Image.Image.convert
                     img = img.convert()
@@ -426,7 +426,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 (width, height) = img.size
                 nw = self.config.epub_max_image_width
                 if width > nw:
-                    nh = (height * nw) / width
+                    nh = round((height * nw) / width)
                     img = img.resize((nw, nh), Image.BICUBIC)
             try:
                 img.save(path.join(self.outdir, self.imagedir, dest))
@@ -451,8 +451,14 @@ class EpubBuilder(StandaloneHTMLBuilder):
     def copy_download_files(self) -> None:
         pass
 
-    def handle_page(self, pagename: str, addctx: dict, templatename: str = 'page.html',
-                    outfilename: str | None = None, event_arg: Any = None) -> None:
+    def handle_page(
+        self,
+        pagename: str,
+        addctx: dict[str, Any],
+        templatename: str = 'page.html',
+        outfilename: str | None = None,
+        event_arg: Any = None,
+    ) -> None:
         """Create a rendered page.
 
         This method is overwritten for genindex pages in order to fix href link
@@ -481,6 +487,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """Create a dictionary with all metadata for the content.opf
         file properly escaped.
         """
+        if (source_date_epoch := os.getenv('SOURCE_DATE_EPOCH')) is not None:
+            time_tuple = time.gmtime(int(source_date_epoch))
+        else:
+            time_tuple = time.gmtime()
+
         metadata: dict[str, Any] = {}
         metadata['title'] = html.escape(self.config.epub_title)
         metadata['author'] = html.escape(self.config.epub_author)
@@ -490,7 +501,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         metadata['copyright'] = html.escape(self.config.epub_copyright)
         metadata['scheme'] = html.escape(self.config.epub_scheme)
         metadata['id'] = html.escape(self.config.epub_identifier)
-        metadata['date'] = html.escape(format_date("%Y-%m-%d", language='en'))
+        metadata['date'] = html.escape(time.strftime('%Y-%m-%d', time_tuple))
         metadata['manifest_items'] = []
         metadata['spines'] = []
         metadata['guides'] = []
@@ -504,21 +515,26 @@ class EpubBuilder(StandaloneHTMLBuilder):
         metadata = self.content_metadata()
 
         # files
-        if not self.outdir.endswith(os.sep):
-            self.outdir += os.sep
-        olen = len(self.outdir)
         self.files: list[str] = []
-        self.ignored_files = ['.buildinfo', 'mimetype', 'content.opf',
-                              'toc.ncx', 'META-INF/container.xml',
-                              'Thumbs.db', 'ehthumbs.db', '.DS_Store',
-                              'nav.xhtml', self.config.epub_basename + '.epub'] + \
-            self.config.epub_exclude_files
+        self.ignored_files = [
+            '.buildinfo',
+            'mimetype',
+            'content.opf',
+            'toc.ncx',
+            'META-INF/container.xml',
+            'Thumbs.db',
+            'ehthumbs.db',
+            '.DS_Store',
+            'nav.xhtml',
+            self.config.epub_basename + '.epub',
+            *self.config.epub_exclude_files,
+        ]
         if not self.use_index:
             self.ignored_files.append('genindex' + self.out_suffix)
         for root, dirs, files in os.walk(self.outdir):
             dirs.sort()
             for fn in sorted(files):
-                filename = path.join(root, fn)[olen:]
+                filename = relpath(path.join(root, fn), self.outdir)
                 if filename in self.ignored_files:
                     continue
                 ext = path.splitext(filename)[-1]
@@ -605,7 +621,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                                             html.escape(self.refnodes[0]['refuri'])))
 
         # write the project file
-        copy_asset_file(path.join(self.template_dir, 'content.opf_t'), self.outdir, metadata)
+        copy_asset_file(path.join(self.template_dir, 'content.opf.jinja'), self.outdir, metadata)  # NoQA: E501
 
     def new_navpoint(self, node: dict[str, Any], level: int, incr: bool = True) -> NavPoint:
         """Create a new entry in the toc from the node at given level."""
@@ -688,7 +704,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         navpoints = self.build_navpoints(refnodes)
         level = max(item['level'] for item in self.refnodes)
         level = min(level, self.config.epub_tocdepth)
-        copy_asset_file(path.join(self.template_dir, 'toc.ncx_t'), self.outdir,
+        copy_asset_file(path.join(self.template_dir, 'toc.ncx.jinja'), self.outdir,
                         self.toc_metadata(level, navpoints))
 
     def build_epub(self) -> None:
