@@ -8,23 +8,25 @@ from __future__ import annotations
 
 import copy
 from abc import ABC, abstractmethod
-from typing import TYPE_CHECKING, Any, Callable, Iterable, NamedTuple, Optional, cast
+from typing import TYPE_CHECKING, Any, Callable, NamedTuple, Optional, cast
 
-from docutils import nodes
 from docutils.nodes import Element, Node, system_message
-from docutils.parsers.rst.states import Inliner
 
-from sphinx.addnodes import pending_xref
 from sphinx.errors import SphinxError
 from sphinx.locale import _
-from sphinx.roles import XRefRole
-from sphinx.util.typing import RoleFunction
 
 if TYPE_CHECKING:
-    from docutils.parsers.rst import Directive
+    from collections.abc import Iterable, Sequence
 
+    from docutils import nodes
+    from docutils.parsers.rst import Directive
+    from docutils.parsers.rst.states import Inliner
+
+    from sphinx.addnodes import pending_xref
     from sphinx.builders import Builder
     from sphinx.environment import BuildEnvironment
+    from sphinx.roles import XRefRole
+    from sphinx.util.typing import RoleFunction
 
 
 class ObjType:
@@ -91,7 +93,7 @@ class Index(ABC):
     shortname: str | None = None
 
     def __init__(self, domain: Domain) -> None:
-        if self.name is None or self.localname is None:
+        if not self.name or self.localname is None:
             raise SphinxError('Index subclass %s has no valid name or localname'
                               % self.__class__.__name__)
         self.domain = domain
@@ -196,7 +198,7 @@ class Domain:
     #: data value for a fresh environment
     initial_data: dict = {}
     #: data value
-    data: dict
+    data: dict[str, Any]
     #: data version, bump this when the format of `self.data` changes
     data_version = 0
 
@@ -262,10 +264,11 @@ class Domain:
         fullname = f'{self.name}:{name}'
 
         def role_adapter(typ: str, rawtext: str, text: str, lineno: int,
-                         inliner: Inliner, options: dict = {}, content: list[str] = [],
+                         inliner: Inliner, options: dict | None = None,
+                         content: Sequence[str] = (),
                          ) -> tuple[list[Node], list[system_message]]:
             return self.roles[name](fullname, rawtext, text, lineno,
-                                    inliner, options, content)
+                                    inliner, options or {}, content)
         self._role_cache[name] = role_adapter
         return role_adapter
 
@@ -293,7 +296,7 @@ class Domain:
         """Remove traces of a document in the domain-specific inventories."""
         pass
 
-    def merge_domaindata(self, docnames: list[str], otherdata: dict) -> None:
+    def merge_domaindata(self, docnames: list[str], otherdata: dict[str, Any]) -> None:
         """Merge in data regarding *docnames* from a different domaindata
         inventory (coming from a subprocess in parallel builds).
         """
