@@ -3,21 +3,25 @@
 from __future__ import annotations
 
 import re
-from collections import OrderedDict
-from typing import Any, Iterable, cast
+from collections.abc import Iterable
+from typing import TYPE_CHECKING, Any, cast
 
 from docutils import nodes
-from docutils.nodes import Element
 
 import sphinx
 from sphinx import addnodes
-from sphinx.application import Sphinx
 from sphinx.util import inspect
-from sphinx.util.typing import stringify_annotation
+from sphinx.util.typing import ExtensionMetadata, stringify_annotation
+
+if TYPE_CHECKING:
+    from docutils.nodes import Element
+
+    from sphinx.application import Sphinx
+    from sphinx.ext.autodoc import Options
 
 
 def record_typehints(app: Sphinx, objtype: str, name: str, obj: Any,
-                     options: dict, args: str, retann: str) -> None:
+                     options: Options, args: str, retann: str) -> None:
     """Record type hints to env object."""
     if app.config.autodoc_typehints_format == 'short':
         mode = 'smart'
@@ -27,13 +31,13 @@ def record_typehints(app: Sphinx, objtype: str, name: str, obj: Any,
     try:
         if callable(obj):
             annotations = app.env.temp_data.setdefault('annotations', {})
-            annotation = annotations.setdefault(name, OrderedDict())
+            annotation = annotations.setdefault(name, {})
             sig = inspect.signature(obj, type_aliases=app.config.autodoc_type_aliases)
             for param in sig.parameters.values():
                 if param.annotation is not param.empty:
-                    annotation[param.name] = stringify_annotation(param.annotation, mode)
+                    annotation[param.name] = stringify_annotation(param.annotation, mode)  # type: ignore[arg-type]
             if sig.return_annotation is not sig.empty:
-                annotation['return'] = stringify_annotation(sig.return_annotation, mode)
+                annotation['return'] = stringify_annotation(sig.return_annotation, mode)  # type: ignore[arg-type]
     except (TypeError, ValueError):
         pass
 
@@ -47,7 +51,7 @@ def merge_typehints(app: Sphinx, domain: str, objtype: str, contentnode: Element
     try:
         signature = cast(addnodes.desc_signature, contentnode.parent[0])
         if signature['module']:
-            fullname = '.'.join([signature['module'], signature['fullname']])
+            fullname = f'{signature["module"]}.{signature["fullname"]}'
         else:
             fullname = signature['fullname']
     except KeyError:
@@ -69,11 +73,11 @@ def merge_typehints(app: Sphinx, domain: str, objtype: str, contentnode: Element
                     modify_field_list(field_list, annotations[fullname])
             elif app.config.autodoc_typehints_description_target == "documented_params":
                 augment_descriptions_with_types(
-                    field_list, annotations[fullname], force_rtype=True
+                    field_list, annotations[fullname], force_rtype=True,
                 )
             else:
                 augment_descriptions_with_types(
-                    field_list, annotations[fullname], force_rtype=False
+                    field_list, annotations[fullname], force_rtype=False,
                 )
 
 
@@ -153,7 +157,7 @@ def modify_field_list(node: nodes.field_list, annotations: dict[str, str],
 def augment_descriptions_with_types(
     node: nodes.field_list,
     annotations: dict[str, str],
-    force_rtype: bool
+    force_rtype: bool,
 ) -> None:
     fields = cast(Iterable[nodes.field], node)
     has_description: set[str] = set()
@@ -205,7 +209,7 @@ def augment_descriptions_with_types(
             node += field
 
 
-def setup(app: Sphinx) -> dict[str, Any]:
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.connect('autodoc-process-signature', record_typehints)
     app.connect('object-description-transform', merge_typehints)
 
