@@ -8,20 +8,24 @@ This requires the MathJax JavaScript library on your webserver/computer.
 from __future__ import annotations
 
 import json
-from typing import Any, cast
+from typing import TYPE_CHECKING, Any, cast
 
 from docutils import nodes
 
 import sphinx
-from sphinx.application import Sphinx
+from sphinx.builders.html import StandaloneHTMLBuilder
 from sphinx.domains.math import MathDomain
 from sphinx.errors import ExtensionError
 from sphinx.locale import _
 from sphinx.util.math import get_node_equation_number
-from sphinx.writers.html import HTML5Translator
+
+if TYPE_CHECKING:
+    from sphinx.application import Sphinx
+    from sphinx.util.typing import ExtensionMetadata
+    from sphinx.writers.html import HTML5Translator
 
 # more information for mathjax secure url is here:
-# https://docs.mathjax.org/en/latest/start.html#secure-access-to-the-cdn
+# https://docs.mathjax.org/en/latest/web/start.html#using-mathjax-from-a-content-delivery-network-cdn
 MATHJAX_URL = 'https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js'
 
 logger = sphinx.util.logging.getLogger(__name__)
@@ -46,7 +50,7 @@ def html_visit_displaymath(self: HTML5Translator, node: nodes.math_block) -> Non
     if node['number']:
         number = get_node_equation_number(self, node)
         self.body.append('<span class="eqno">(%s)' % number)
-        self.add_permalink_ref(node, _('Permalink to this equation'))
+        self.add_permalink_ref(node, _('Link to this equation'))
         self.body.append('</span>')
     self.body.append(self.builder.config.mathjax_display[0])
     parts = [prt for prt in node.astext().split('\n\n') if prt.strip()]
@@ -75,10 +79,11 @@ def install_mathjax(app: Sphinx, pagename: str, templatename: str, context: dict
     ):
         return
     if not app.config.mathjax_path:
-        raise ExtensionError('mathjax_path config value must be set for the '
-                             'mathjax extension to work')
+        msg = 'mathjax_path config value must be set for the mathjax extension to work'
+        raise ExtensionError(msg)
 
     domain = cast(MathDomain, app.env.get_domain('math'))
+    builder = cast(StandaloneHTMLBuilder, app.builder)
     if app.registry.html_assets_policy == 'always' or domain.has_equations(pagename):
         # Enable mathjax only if equations exists
         if app.config.mathjax2_config:
@@ -87,10 +92,10 @@ def install_mathjax(app: Sphinx, pagename: str, templatename: str, context: dict
                     'mathjax_config/mathjax2_config does not work '
                     'for the current MathJax version, use mathjax3_config instead')
             body = 'MathJax.Hub.Config(%s)' % json.dumps(app.config.mathjax2_config)
-            app.add_js_file(None, type='text/x-mathjax-config', body=body)
+            builder.add_js_file('', type='text/x-mathjax-config', body=body)
         if app.config.mathjax3_config:
             body = 'window.MathJax = %s' % json.dumps(app.config.mathjax3_config)
-            app.add_js_file(None, body=body)
+            builder.add_js_file('', body=body)
 
         options = {}
         if app.config.mathjax_options:
@@ -102,10 +107,10 @@ def install_mathjax(app: Sphinx, pagename: str, templatename: str, context: dict
             else:
                 # Load other MathJax via "async" method
                 options['async'] = 'async'
-        app.add_js_file(app.config.mathjax_path, **options)
+        builder.add_js_file(app.config.mathjax_path, **options)
 
 
-def setup(app: Sphinx) -> dict[str, Any]:
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.add_html_math_renderer('mathjax',
                                (html_visit_math, None),
                                (html_visit_displaymath, None))
