@@ -1,19 +1,25 @@
 """Create a full-text search index for offline search."""
+from __future__ import annotations
+
+import dataclasses
+import functools
 import html
 import json
 import pickle
 import re
-import warnings
 from importlib import import_module
 from os import path
-from typing import IO, Any, Dict, Iterable, List, Optional, Set, Tuple, Type
+from typing import IO, TYPE_CHECKING, Any
 
 from docutils import nodes
 from docutils.nodes import Element, Node
 
 from sphinx import addnodes, package_dir
-from sphinx.deprecation import RemovedInSphinx70Warning
 from sphinx.environment import BuildEnvironment
+from sphinx.util.index_entries import split_index_msg
+
+if TYPE_CHECKING:
+    from collections.abc import Iterable
 
 
 class SearchLanguage:
@@ -46,11 +52,11 @@ class SearchLanguage:
        This class is used to preprocess search word which Sphinx HTML readers
        type, before searching index. Default implementation does nothing.
     """
-    lang: str = None
-    language_name: str = None
-    stopwords: Set[str] = set()
+    lang: str = ''
+    language_name: str = ''
+    stopwords: set[str] = set()
     js_splitter_code: str = ""
-    js_stemmer_rawcode: str = None
+    js_stemmer_rawcode: str = ''
     js_stemmer_code = """
 /**
  * Dummy stemmer for languages without stemming rules.
@@ -62,18 +68,18 @@ var Stemmer = function() {
 }
 """
 
-    _word_re = re.compile(r'(?u)\w+')
+    _word_re = re.compile(r'\w+')
 
-    def __init__(self, options: Dict) -> None:
+    def __init__(self, options: dict[str, str]) -> None:
         self.options = options
         self.init(options)
 
-    def init(self, options: Dict) -> None:
+    def init(self, options: dict[str, str]) -> None:
         """
         Initialize the class with the options the user has given.
         """
 
-    def split(self, input: str) -> List[str]:
+    def split(self, input: str) -> list[str]:
         """
         This method splits a sentence into words.  Default splitter splits input
         at white spaces, which should be enough for most languages except CJK
@@ -111,13 +117,13 @@ var Stemmer = function() {
 from sphinx.search.en import SearchEnglish
 
 
-def parse_stop_word(source: str) -> Set[str]:
+def parse_stop_word(source: str) -> set[str]:
     """
     Parse snowball style word list like this:
 
-    * http://snowball.tartarus.org/algorithms/finnish/stop.txt
+    * https://snowball.tartarus.org/algorithms/finnish/stop.txt
     """
-    result: Set[str] = set()
+    result: set[str] = set()
     for line in source.splitlines():
         line = line.split('|')[0]  # remove comment
         result.update(line.split())
@@ -125,7 +131,7 @@ def parse_stop_word(source: str) -> Set[str]:
 
 
 # maps language name to module.class or directly a class
-languages: Dict[str, Any] = {
+languages: dict[str, str | type[SearchLanguage]] = {
     'da': 'sphinx.search.da.SearchDanish',
     'de': 'sphinx.search.de.SearchGerman',
     'en': SearchEnglish,
@@ -156,7 +162,7 @@ class _JavaScriptIndex:
     SUFFIX = ')'
 
     def dumps(self, data: Any) -> str:
-        return self.PREFIX + json.dumps(data) + self.SUFFIX
+        return self.PREFIX + json.dumps(data, sort_keys=True) + self.SUFFIX
 
     def loads(self, s: str) -> Any:
         data = s[len(self.PREFIX):-len(self.SUFFIX)]
@@ -165,16 +171,37 @@ class _JavaScriptIndex:
             raise ValueError('invalid data')
         return json.loads(data)
 
-    def dump(self, data: Any, f: IO) -> None:
+    def dump(self, data: Any, f: IO[str]) -> None:
         f.write(self.dumps(data))
 
-    def load(self, f: IO) -> Any:
+    def load(self, f: IO[str]) -> Any:
         return self.loads(f.read())
 
 
 js_index = _JavaScriptIndex()
 
 
+def _is_meta_keywords(
+    node: nodes.meta,
+    lang: str | None,
+) -> bool:
+    if node.get('name') == 'keywords':
+        meta_lang = node.get('lang')
+        if meta_lang is None:  # lang not specified
+            return True
+        elif meta_lang == lang:  # matched to html_search_language
+            return True
+
+    return False
+
+
+@dataclasses.dataclass
+class WordStore:
+    words: list[str] = dataclasses.field(default_factory=list)
+    titles: list[tuple[str, str | None]] = dataclasses.field(default_factory=list)
+    title_words: list[str] = dataclasses.field(default_factory=list)
+
+
 class WordCollector(nodes.NodeVisitor):
     """
     A special visitor that collects words for the `IndexBuilder`.
@@ -182,21 +209,11 @@ class WordCollector(nodes.NodeVisitor):
 
     def __init__(self, document: nodes.document, lang: SearchLanguage) -> None:
         super().__init__(document)
-        self.found_words: List[str] = []
-        self.found_title_words: List[str] = []
+        self.found_words: list[str] = []
+        self.found_titles: list[tuple[str, str]] = []
+        self.found_title_words: list[str] = []
         self.lang = lang
 
-    def is_meta_keywords(self, node: Element) -> bool:
-        if (isinstance(node, (addnodes.meta, addnodes.docutils_meta)) and
-                node.get('name') == 'keywords'):
-            meta_lang = node.get('lang')
-            if meta_lang is None:  # lang not specified
-                return True
-            elif meta_lang == self.lang.lang:  # matched to html_search_language
-                return True
-
-        return False
-
     def dispatch_visit(self, node: Node) -> None:
         if isinstance(node, nodes.comment):
             raise nodes.SkipNode
@@ -205,16 +222,19 @@ class WordCollector(nodes.NodeVisitor):
                 # Some people might put content in raw HTML that should be searched,
                 # so we just amateurishly strip HTML tags and index the remaining
                 # content
-                nodetext = re.sub(r'(?is)<style.*?</style>', '', node.astext())
-                nodetext = re.sub(r'(?is)<script.*?</script>', '', nodetext)
+                nodetext = re.sub(r'<style.*?</style>', '', node.astext(), flags=re.IGNORECASE|re.DOTALL)
+                nodetext = re.sub(r'<script.*?</script>', '', nodetext, flags=re.IGNORECASE|re.DOTALL)
                 nodetext = re.sub(r'<[^<]+?>', '', nodetext)
                 self.found_words.extend(self.lang.split(nodetext))
             raise nodes.SkipNode
         elif isinstance(node, nodes.Text):
             self.found_words.extend(self.lang.split(node.astext()))
         elif isinstance(node, nodes.title):
-            self.found_title_words.extend(self.lang.split(node.astext()))
-        elif isinstance(node, Element) and self.is_meta_keywords(node):
+            title = node.astext()
+            ids = node.parent['ids']
+            self.found_titles.append((title, ids[0] if ids else None))
+            self.found_title_words.extend(self.lang.split(title))
+        elif isinstance(node, Element) and _is_meta_keywords(node, self.lang.lang):  # type: ignore[arg-type]
             keywords = node['content']
             keywords = [keyword.strip() for keyword in keywords.split(',')]
             self.found_words.extend(keywords)
@@ -230,19 +250,26 @@ class IndexBuilder:
         'pickle':   pickle
     }
 
-    def __init__(self, env: BuildEnvironment, lang: str, options: Dict, scoring: str) -> None:
+    def __init__(self, env: BuildEnvironment, lang: str, options: dict[str, str], scoring: str) -> None:
         self.env = env
-        self._titles: Dict[str, str] = {}           # docname -> title
-        self._filenames: Dict[str, str] = {}        # docname -> filename
-        self._mapping: Dict[str, Set[str]] = {}     # stemmed word -> set(docname)
+        # docname -> title
+        self._titles: dict[str, str | None] = env._search_index_titles
+        # docname -> filename
+        self._filenames: dict[str, str] = env._search_index_filenames
+        # stemmed words -> set(docname)
+        self._mapping: dict[str, set[str]] = env._search_index_mapping
         # stemmed words in titles -> set(docname)
-        self._title_mapping: Dict[str, Set[str]] = {}
-        self._stem_cache: Dict[str, str] = {}       # word -> stemmed word
-        self._objtypes: Dict[Tuple[str, str], int] = {}     # objtype -> index
+        self._title_mapping: dict[str, set[str]] = env._search_index_title_mapping
+        # docname -> all titles in document
+        self._all_titles: dict[str, list[tuple[str, str | None]]] = env._search_index_all_titles
+        # docname -> list(index entry)
+        self._index_entries: dict[str, list[tuple[str, str, str]]] = env._search_index_index_entries
+        # objtype -> index
+        self._objtypes: dict[tuple[str, str], int] = env._search_index_objtypes
         # objtype index -> (domain, type, objname (localized))
-        self._objnames: Dict[int, Tuple[str, str, str]] = {}
+        self._objnames: dict[int, tuple[str, str, str]] = env._search_index_objnames
         # add language-specific SearchLanguage instance
-        lang_class: Type[SearchLanguage] = languages.get(lang)
+        lang_class = languages.get(lang)
 
         # fallback; try again with language-code
         if lang_class is None and '_' in lang:
@@ -252,8 +279,8 @@ class IndexBuilder:
             self.lang: SearchLanguage = SearchEnglish(options)
         elif isinstance(lang_class, str):
             module, classname = lang_class.rsplit('.', 1)
-            lang_class = getattr(import_module(module), classname)
-            self.lang = lang_class(options)
+            lang_class: type[SearchLanguage] = getattr(import_module(module), classname)  # type: ignore[no-redef]
+            self.lang = lang_class(options)  # type: ignore[operator]
         else:
             # it's directly a class (e.g. added by app.add_search_language)
             self.lang = lang_class(options)
@@ -267,11 +294,7 @@ class IndexBuilder:
 
     def load(self, stream: IO, format: Any) -> None:
         """Reconstruct from frozen data."""
-        if format == "jsdump":
-            warnings.warn("format=jsdump is deprecated, use json instead",
-                          RemovedInSphinx70Warning, stacklevel=2)
-            format = self.formats["json"]
-        elif isinstance(format, str):
+        if isinstance(format, str):
             format = self.formats[format]
         frozen = format.load(stream)
         # if an old index is present, we treat it as not existing.
@@ -281,8 +304,15 @@ class IndexBuilder:
         index2fn = frozen['docnames']
         self._filenames = dict(zip(index2fn, frozen['filenames']))
         self._titles = dict(zip(index2fn, frozen['titles']))
+        self._all_titles = {}
 
-        def load_terms(mapping: Dict[str, Any]) -> Dict[str, Set[str]]:
+        for docname in self._titles.keys():
+            self._all_titles[docname] = []
+        for title, doc_tuples in frozen['alltitles'].items():
+            for doc, titleid in doc_tuples:
+                self._all_titles[index2fn[doc]].append((title, titleid))
+
+        def load_terms(mapping: dict[str, Any]) -> dict[str, set[str]]:
             rv = {}
             for k, v in mapping.items():
                 if isinstance(v, int):
@@ -297,17 +327,13 @@ class IndexBuilder:
 
     def dump(self, stream: IO, format: Any) -> None:
         """Dump the frozen index to a stream."""
-        if format == "jsdump":
-            warnings.warn("format=jsdump is deprecated, use json instead",
-                          RemovedInSphinx70Warning, stacklevel=2)
-            format = self.formats["json"]
-        elif isinstance(format, str):
+        if isinstance(format, str):
             format = self.formats[format]
         format.dump(self.freeze(), stream)
 
-    def get_objects(self, fn2index: Dict[str, int]
-                    ) -> Dict[str, List[Tuple[int, int, int, str, str]]]:
-        rv: Dict[str, List[Tuple[int, int, int, str, str]]] = {}
+    def get_objects(self, fn2index: dict[str, int]
+                    ) -> dict[str, list[tuple[int, int, int, str, str]]]:
+        rv: dict[str, list[tuple[int, int, int, str, str]]] = {}
         otypes = self._objtypes
         onames = self._objnames
         for domainname, domain in sorted(self.env.domains.items()):
@@ -342,8 +368,15 @@ class IndexBuilder:
                 plist.append((fn2index[docname], typeindex, prio, shortanchor, name))
         return rv
 
-    def get_terms(self, fn2index: Dict) -> Tuple[Dict[str, List[str]], Dict[str, List[str]]]:
-        rvs: Tuple[Dict[str, List[str]], Dict[str, List[str]]] = ({}, {})
+    def get_terms(self, fn2index: dict[str, int]) -> tuple[dict[str, list[int] | int], dict[str, list[int] | int]]:
+        """
+        Return a mapping of document and title terms to their corresponding sorted document IDs.
+
+        When a term is only found within a single document, then the value for that term will be
+        an integer value.  When a term is found within multiple documents, the value will be a list
+        of integers.
+        """
+        rvs: tuple[dict[str, list[int] | int], dict[str, list[int] | int]] = ({}, {})
         for rv, mapping in zip(rvs, (self._mapping, self._title_mapping)):
             for k, v in mapping.items():
                 if len(v) == 1:
@@ -351,10 +384,10 @@ class IndexBuilder:
                     if fn in fn2index:
                         rv[k] = fn2index[fn]
                 else:
-                    rv[k] = sorted([fn2index[fn] for fn in v if fn in fn2index])
+                    rv[k] = sorted(fn2index[fn] for fn in v if fn in fn2index)
         return rvs
 
-    def freeze(self) -> Dict[str, Any]:
+    def freeze(self) -> dict[str, Any]:
         """Create a usable data structure for serializing."""
         docnames, titles = zip(*sorted(self._titles.items()))
         filenames = [self._filenames.get(docname) for docname in docnames]
@@ -364,23 +397,38 @@ class IndexBuilder:
         objects = self.get_objects(fn2index)  # populates _objtypes
         objtypes = {v: k[0] + ':' + k[1] for (k, v) in self._objtypes.items()}
         objnames = self._objnames
+
+        alltitles: dict[str, list[tuple[int, str | None]]] = {}
+        for docname, titlelist in sorted(self._all_titles.items()):
+            for title, titleid in titlelist:
+                alltitles.setdefault(title, []).append((fn2index[docname], titleid))
+
+        index_entries: dict[str, list[tuple[int, str, bool]]] = {}
+        for docname, entries in self._index_entries.items():
+            for entry, entry_id, main_entry in entries:
+                index_entries.setdefault(entry.lower(), []).append((fn2index[docname], entry_id, main_entry == "main"))
+
         return dict(docnames=docnames, filenames=filenames, titles=titles, terms=terms,
                     objects=objects, objtypes=objtypes, objnames=objnames,
-                    titleterms=title_terms, envversion=self.env.version)
+                    titleterms=title_terms, envversion=self.env.version,
+                    alltitles=alltitles, indexentries=index_entries)
 
     def label(self) -> str:
-        return "%s (code: %s)" % (self.lang.language_name, self.lang.lang)
+        return f"{self.lang.language_name} (code: {self.lang.lang})"
 
     def prune(self, docnames: Iterable[str]) -> None:
         """Remove data for all docnames not in the list."""
         new_titles = {}
+        new_alltitles = {}
         new_filenames = {}
         for docname in docnames:
             if docname in self._titles:
                 new_titles[docname] = self._titles[docname]
+                new_alltitles[docname] = self._all_titles[docname]
                 new_filenames[docname] = self._filenames[docname]
         self._titles = new_titles
         self._filenames = new_filenames
+        self._all_titles = new_alltitles
         for wordnames in self._mapping.values():
             wordnames.intersection_update(docnames)
         for wordnames in self._title_mapping.values():
@@ -391,35 +439,91 @@ class IndexBuilder:
         self._titles[docname] = title
         self._filenames[docname] = filename
 
-        visitor = WordCollector(doctree, self.lang)
-        doctree.walk(visitor)
+        word_store = self._word_collector(doctree)
 
-        # memoize self.lang.stem
-        def stem(word: str) -> str:
-            try:
-                return self._stem_cache[word]
-            except KeyError:
-                self._stem_cache[word] = self.lang.stem(word).lower()
-                return self._stem_cache[word]
         _filter = self.lang.word_filter
+        _stem = self.lang.stem
 
-        for word in visitor.found_title_words:
+        # memoise self.lang.stem
+        @functools.cache
+        def stem(word_to_stem: str) -> str:
+            return _stem(word_to_stem).lower()
+
+        self._all_titles[docname] = word_store.titles
+
+        for word in word_store.title_words:
+            # add stemmed and unstemmed as the stemmer must not remove words
+            # from search index.
             stemmed_word = stem(word)
             if _filter(stemmed_word):
                 self._title_mapping.setdefault(stemmed_word, set()).add(docname)
-            elif _filter(word): # stemmer must not remove words from search index
+            elif _filter(word):
                 self._title_mapping.setdefault(word, set()).add(docname)
 
-        for word in visitor.found_words:
+        for word in word_store.words:
+            # add stemmed and unstemmed as the stemmer must not remove words
+            # from search index.
             stemmed_word = stem(word)
-            # again, stemmer must not remove words from search index
             if not _filter(stemmed_word) and _filter(word):
                 stemmed_word = word
-            already_indexed = docname in self._title_mapping.get(stemmed_word, set())
+            already_indexed = docname in self._title_mapping.get(stemmed_word, ())
             if _filter(stemmed_word) and not already_indexed:
                 self._mapping.setdefault(stemmed_word, set()).add(docname)
 
-    def context_for_searchtool(self) -> Dict[str, Any]:
+        # find explicit entries within index directives
+        _index_entries: set[tuple[str, str, str]] = set()
+        for node in doctree.findall(addnodes.index):
+            for entry_type, value, target_id, main, _category_key in node['entries']:
+                try:
+                    result = split_index_msg(entry_type, value)
+                except ValueError:
+                    pass
+                else:
+                    target_id = target_id or ''
+                    if entry_type in {'see', 'seealso'}:
+                        _index_entries.add((result[0], target_id, main))
+                    _index_entries |= {(x, target_id, main) for x in result}
+
+        self._index_entries[docname] = sorted(_index_entries)
+
+    def _word_collector(self, doctree: nodes.document) -> WordStore:
+        def _visit_nodes(node):
+            if isinstance(node, nodes.comment):
+                return
+            elif isinstance(node, nodes.raw):
+                if 'html' in node.get('format', '').split():
+                    # Some people might put content in raw HTML that should be searched,
+                    # so we just amateurishly strip HTML tags and index the remaining
+                    # content
+                    nodetext = re.sub(r'<style.*?</style>', '', node.astext(),
+                                      flags=re.IGNORECASE | re.DOTALL)
+                    nodetext = re.sub(r'<script.*?</script>', '', nodetext,
+                                      flags=re.IGNORECASE | re.DOTALL)
+                    nodetext = re.sub(r'<[^<]+?>', '', nodetext)
+                    word_store.words.extend(split(nodetext))
+                return
+            elif (isinstance(node, nodes.meta)
+                  and _is_meta_keywords(node, language)):
+                keywords = [keyword.strip() for keyword in node['content'].split(',')]
+                word_store.words.extend(keywords)
+            elif isinstance(node, nodes.Text):
+                word_store.words.extend(split(node.astext()))
+            elif isinstance(node, nodes.title):
+                title, is_main_title = node.astext(), len(word_store.titles) == 0
+                ids = node.parent['ids']
+                title_node_id = None if is_main_title else ids[0] if ids else None
+                word_store.titles.append((title, title_node_id))
+                word_store.title_words.extend(split(title))
+            for child in node.children:
+                _visit_nodes(child)
+
+        word_store = WordStore()
+        split = self.lang.split
+        language = self.lang.lang
+        _visit_nodes(doctree)
+        return word_store
+
+    def context_for_searchtool(self) -> dict[str, Any]:
         if self.lang.js_splitter_code:
             js_splitter_code = self.lang.js_splitter_code
         else:
@@ -432,7 +536,7 @@ class IndexBuilder:
             'search_word_splitter_code': js_splitter_code,
         }
 
-    def get_js_stemmer_rawcodes(self) -> List[str]:
+    def get_js_stemmer_rawcodes(self) -> list[str]:
         """Returns a list of non-minified stemmer JS files to copy."""
         if self.lang.js_stemmer_rawcode:
             return [
@@ -442,7 +546,7 @@ class IndexBuilder:
         else:
             return []
 
-    def get_js_stemmer_rawcode(self) -> Optional[str]:
+    def get_js_stemmer_rawcode(self) -> str | None:
         return None
 
     def get_js_stemmer_code(self) -> str:
