@@ -2,14 +2,14 @@
 
 from __future__ import annotations
 
-from typing import Dict
+from typing import TYPE_CHECKING, Dict
 
 import snowballstemmer
 
 from sphinx.search import SearchLanguage, parse_stop_word
 
 russian_stopwords = parse_stop_word('''
-| source: http://snowball.tartarus.org/algorithms/russian/stop.txt
+| source: https://snowball.tartarus.org/algorithms/russian/stop.txt
 и              | and
 в              | in/into
 во             | alternative form
@@ -244,7 +244,7 @@ class SearchRussian(SearchLanguage):
     js_stemmer_rawcode = 'russian-stemmer.js'
     stopwords = russian_stopwords
 
-    def init(self, options: dict) -> None:
+    def init(self, options: dict[str, str]) -> None:
         self.stemmer = snowballstemmer.stemmer('russian')
 
     def stem(self, word: str) -> str:
