@@ -6,7 +6,7 @@ import os
 import re
 from datetime import datetime, timezone
 from os import path
-from typing import TYPE_CHECKING, Callable, Generator, NamedTuple
+from typing import TYPE_CHECKING, NamedTuple
 
 import babel.dates
 from babel.messages.mofile import write_mo
@@ -18,8 +18,41 @@ from sphinx.util import logging
 from sphinx.util.osutil import SEP, canon_path, relpath
 
 if TYPE_CHECKING:
+    import datetime as dt
+    from collections.abc import Iterator
+    from typing import Protocol, Union
+
+    from babel.core import Locale
+
     from sphinx.environment import BuildEnvironment
 
+    class DateFormatter(Protocol):
+        def __call__(  # NoQA: E704
+            self,
+            date: dt.date | None = ...,
+            format: str = ...,
+            locale: str | Locale | None = ...,
+        ) -> str: ...
+
+    class TimeFormatter(Protocol):
+        def __call__(  # NoQA: E704
+            self,
+            time: dt.time | dt.datetime | float | None = ...,
+            format: str = ...,
+            tzinfo: dt.tzinfo | None = ...,
+            locale: str | Locale | None = ...,
+        ) -> str: ...
+
+    class DatetimeFormatter(Protocol):
+        def __call__(  # NoQA: E704
+            self,
+            datetime: dt.date | dt.time | float | None = ...,
+            format: str = ...,
+            tzinfo: dt.tzinfo | None = ...,
+            locale: str | Locale | None = ...,
+        ) -> str: ...
+
+    Formatter = Union[DateFormatter, TimeFormatter, DatetimeFormatter]
 
 logger = logging.getLogger(__name__)
 
@@ -71,7 +104,7 @@ class CatalogInfo(LocaleFileInfoBase):
 class CatalogRepository:
     """A repository for message catalogs."""
 
-    def __init__(self, basedir: str, locale_dirs: list[str],
+    def __init__(self, basedir: str | os.PathLike[str], locale_dirs: list[str],
                  language: str, encoding: str) -> None:
         self.basedir = basedir
         self._locale_dirs = locale_dirs
@@ -79,7 +112,7 @@ class CatalogRepository:
         self.encoding = encoding
 
     @property
-    def locale_dirs(self) -> Generator[str, None, None]:
+    def locale_dirs(self) -> Iterator[str]:
         if not self.language:
             return
 
@@ -89,17 +122,16 @@ class CatalogRepository:
             if path.exists(locale_path):
                 yield locale_dir
             else:
-                logger.verbose(__('locale_dir %s does not exists'), locale_path)
+                logger.verbose(__('locale_dir %s does not exist'), locale_path)
 
     @property
-    def pofiles(self) -> Generator[tuple[str, str], None, None]:
+    def pofiles(self) -> Iterator[tuple[str, str]]:
         for locale_dir in self.locale_dirs:
             basedir = path.join(locale_dir, self.language, 'LC_MESSAGES')
             for root, dirnames, filenames in os.walk(basedir):
                 # skip dot-directories
-                for dirname in dirnames:
-                    if dirname.startswith('.'):
-                        dirnames.remove(dirname)
+                for dirname in [d for d in dirnames if d.startswith('.')]:
+                    dirnames.remove(dirname)
 
                 for filename in filenames:
                     if filename.endswith('.po'):
@@ -107,7 +139,7 @@ class CatalogRepository:
                         yield basedir, relpath(fullpath, basedir)
 
     @property
-    def catalogs(self) -> Generator[CatalogInfo, None, None]:
+    def catalogs(self) -> Iterator[CatalogInfo]:
         for basedir, filename in self.pofiles:
             domain = canon_path(path.splitext(filename)[0])
             yield CatalogInfo(basedir, domain, self.encoding)
@@ -168,7 +200,7 @@ date_format_re = re.compile('(%s)' % '|'.join(date_format_mappings))
 
 
 def babel_format_date(date: datetime, format: str, locale: str,
-                      formatter: Callable = babel.dates.format_date) -> str:
+                      formatter: Formatter = babel.dates.format_date) -> str:
     # Check if we have the tzinfo attribute. If not we cannot do any time
     # related formats.
     if not hasattr(date, 'tzinfo'):
@@ -206,6 +238,7 @@ def format_date(
             # Check if we have to use a different babel formatter then
             # format_datetime, because we only want to format a date
             # or a time.
+            function: Formatter
             if token == '%x':
                 function = babel.dates.format_date
             elif token == '%X':
@@ -221,24 +254,25 @@ def format_date(
     return "".join(result)
 
 
-def get_image_filename_for_language(filename: str, env: BuildEnvironment) -> str:
-    filename_format = env.config.figure_language_filename
-    d = {}
-    d['root'], d['ext'] = path.splitext(filename)
-    dirname = path.dirname(d['root'])
-    if dirname and not dirname.endswith(path.sep):
-        dirname += path.sep
+def get_image_filename_for_language(
+    filename: str | os.PathLike[str],
+    env: BuildEnvironment,
+) -> str:
+    root, ext = path.splitext(filename)
+    dirname = path.dirname(root)
     docpath = path.dirname(env.docname)
-    if docpath and not docpath.endswith(path.sep):
-        docpath += path.sep
-    d['path'] = dirname
-    d['basename'] = path.basename(d['root'])
-    d['docpath'] = docpath
-    d['language'] = env.config.language
     try:
-        return filename_format.format(**d)
+        return env.config.figure_language_filename.format(
+            root=root,
+            ext=ext,
+            path=dirname and dirname + SEP,
+            basename=path.basename(root),
+            docpath=docpath and docpath + SEP,
+            language=env.config.language,
+        )
     except KeyError as exc:
-        raise SphinxError('Invalid figure_language_filename: %r' % exc) from exc
+        msg = f'Invalid figure_language_filename: {exc!r}'
+        raise SphinxError(msg) from exc
 
 
 def search_image_for_language(filename: str, env: BuildEnvironment) -> str:
