@@ -24,6 +24,7 @@ mime_suffixes = {
     '.svg': 'image/svg+xml',
     '.svgz': 'image/svg+xml',
     '.ai': 'application/illustrator',
+    '.webp': 'image/webp',
 }
 _suffix_from_mime = {v: k for k, v in reversed(mime_suffixes.items())}
 
@@ -142,4 +143,5 @@ def _image_type_from_file(filename: PathLike[str] | str) -> str:
     if header.startswith(b'RIFF') and header[8:12] == b'WEBP':
         return 'webp'
 
-    raise ValueError('Could not detect image type!')
+    msg = 'Could not detect image type!'
+    raise ValueError(msg)
