@@ -4,10 +4,13 @@ from __future__ import annotations
 
 import os.path
 import re
-from typing import Callable, Iterable, Iterator
+from typing import TYPE_CHECKING, Callable
 
 from sphinx.util.osutil import canon_path, path_stabilize
 
+if TYPE_CHECKING:
+    from collections.abc import Iterable, Iterator
+
 
 def _translate_pattern(pat: str) -> str:
     """Translate a shell-style glob pattern to a regular expression.
@@ -88,7 +91,8 @@ _pat_cache: dict[str, re.Pattern[str]] = {}
 
 def patmatch(name: str, pat: str) -> re.Match[str] | None:
     """Return if name matches the regular expression (pattern)
-    ``pat```. Adapted from fnmatch module."""
+    ``pat```. Adapted from fnmatch module.
+    """
     if pat not in _pat_cache:
         _pat_cache[pat] = re.compile(_translate_pattern(pat))
     return _pat_cache[pat].match(name)
@@ -107,7 +111,7 @@ def patfilter(names: Iterable[str], pat: str) -> list[str]:
 
 
 def get_matching_files(
-    dirname: str,
+    dirname: str | os.PathLike[str],
     include_patterns: Iterable[str] = ("**",),
     exclude_patterns: Iterable[str] = (),
 ) -> Iterator[str]:
