@@ -5,11 +5,11 @@ import math
 import os
 import re
 import textwrap
+from collections.abc import Iterable, Iterator, Sequence
 from itertools import chain, groupby
-from typing import TYPE_CHECKING, Any, Generator, Iterable, cast
+from typing import TYPE_CHECKING, Any, cast
 
 from docutils import nodes, writers
-from docutils.nodes import Element, Text
 from docutils.utils import column_width
 
 from sphinx import addnodes
@@ -17,6 +17,8 @@ from sphinx.locale import _, admonitionlabels
 from sphinx.util.docutils import SphinxTranslator
 
 if TYPE_CHECKING:
+    from docutils.nodes import Element, Text
+
     from sphinx.builders.text import TextBuilder
 
 
@@ -24,6 +26,7 @@ class Cell:
     """Represents a cell in a table.
     It can span multiple columns or multiple lines.
     """
+
     def __init__(self, text: str = "", rowspan: int = 1, colspan: int = 1) -> None:
         self.text = text
         self.wrapped: list[str] = []
@@ -91,6 +94,7 @@ class Table:
        +--------+--------+
 
     """
+
     def __init__(self, colwidth: list[int] | None = None) -> None:
         self.lines: list[list[Cell]] = []
         self.separator = 0
@@ -146,7 +150,7 @@ class Table:
                 line.append(Cell())
 
     def __repr__(self) -> str:
-        return "\n".join(repr(line) for line in self.lines)
+        return "\n".join(map(repr, self.lines))
 
     def cell_width(self, cell: Cell, source: list[int]) -> int:
         """Give the cell width, according to the given source (either
@@ -154,14 +158,15 @@ class Table:
         This takes into account cells spanning multiple columns.
         """
         if cell.row is None or cell.col is None:
-            raise ValueError('Cell co-ordinates have not been set')
+            msg = 'Cell co-ordinates have not been set'
+            raise ValueError(msg)
         width = 0
         for i in range(self[cell.row, cell.col].colspan):
             width += source[cell.col + i]
         return width + (cell.colspan - 1) * 3
 
     @property
-    def cells(self) -> Generator[Cell, None, None]:
+    def cells(self) -> Iterator[Cell]:
         seen: set[Cell] = set()
         for line in self.lines:
             for cell in line:
@@ -179,7 +184,8 @@ class Table:
             if not cell.wrapped:
                 continue
             if cell.row is None or cell.col is None:
-                raise ValueError('Cell co-ordinates have not been set')
+                msg = 'Cell co-ordinates have not been set'
+                raise ValueError(msg)
             width = math.ceil(max(column_width(x) for x in cell.wrapped) / cell.colspan)
             for col in range(cell.col, cell.col + cell.colspan):
                 self.measured_widths[col] = max(self.measured_widths[col], width)
@@ -258,9 +264,8 @@ class TextWrapper(textwrap.TextWrapper):
         r'(?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w))')   # em-dash
 
     def _wrap_chunks(self, chunks: list[str]) -> list[str]:
-        """_wrap_chunks(chunks : [string]) -> [string]
+        """The original _wrap_chunks uses len() to calculate width.
 
-        The original _wrap_chunks uses len() to calculate width.
         This method respects wide/fullwidth characters for width adjustment.
         """
         lines: list[str] = []
@@ -305,10 +310,7 @@ class TextWrapper(textwrap.TextWrapper):
         return lines
 
     def _break_word(self, word: str, space_left: int) -> tuple[str, str]:
-        """_break_word(word : string, space_left : int) -> (string, string)
-
-        Break line by unicode width instead of len(word).
-        """
+        """Break line by unicode width instead of len(word)."""
         total = 0
         for i, c in enumerate(word):
             total += column_width(c)
@@ -317,9 +319,8 @@ class TextWrapper(textwrap.TextWrapper):
         return word, ''
 
     def _split(self, text: str) -> list[str]:
-        """_split(text : string) -> [string]
+        """Override original method that only split by 'wordsep_re'.
 
-        Override original method that only split by 'wordsep_re'.
         This '_split' splits wide-characters into chunks by one character.
         """
         def split(t: str) -> list[str]:
@@ -335,12 +336,7 @@ class TextWrapper(textwrap.TextWrapper):
 
     def _handle_long_word(self, reversed_chunks: list[str], cur_line: list[str],
                           cur_len: int, width: int) -> None:
-        """_handle_long_word(chunks : [string],
-                             cur_line : [string],
-                             cur_len : int, width : int)
-
-        Override original method for using self._break_word() instead of slice.
-        """
+        """Override original method for using self._break_word() instead of slice."""
         space_left = max(width - cur_len, 1)
         if self.break_long_words:
             l, r = self._break_word(reversed_chunks[-1], space_left)
@@ -415,7 +411,7 @@ class TextTranslator(SphinxTranslator):
         self.stateindent.append(indent)
 
     def end_state(
-        self, wrap: bool = True, end: list[str] | None = [''], first: str | None = None,
+        self, wrap: bool = True, end: Sequence[str] | None = ('',), first: str | None = None,
     ) -> None:
         content = self.states.pop()
         maxindent = sum(self.stateindent)
@@ -435,10 +431,10 @@ class TextTranslator(SphinxTranslator):
             result.append((indent, res))
         for itemindent, item in content:
             if itemindent == -1:
-                toformat.append(item)  # type: ignore
+                toformat.append(item)  # type: ignore[arg-type]
             else:
                 do_format()
-                result.append((indent + itemindent, item))  # type: ignore
+                result.append((indent + itemindent, item))  # type: ignore[arg-type]
                 toformat = []
         do_format()
         if first is not None and result:
@@ -520,7 +516,7 @@ class TextTranslator(SphinxTranslator):
         else:
             char = '^'
         text = ''
-        text = ''.join(x[1] for x in self.states.pop() if x[0] == -1)  # type: ignore
+        text = ''.join(x[1] for x in self.states.pop() if x[0] == -1)  # type: ignore[misc]
         if self.add_secnumbers:
             text = self.get_section_number_string(node) + text
         self.stateindent.pop()
@@ -689,8 +685,8 @@ class TextTranslator(SphinxTranslator):
         self.visit_desc_parameter(node)
 
     def visit_desc_optional(self, node: Element) -> None:
-        self.params_left_at_level = sum([isinstance(c, addnodes.desc_parameter)
-                                         for c in node.children])
+        self.params_left_at_level = sum(isinstance(c, addnodes.desc_parameter)
+                                        for c in node.children)
         self.optional_param_level += 1
         self.max_optional_param_level = self.optional_param_level
         if self.multi_line_parameter_list:
@@ -753,10 +749,8 @@ class TextTranslator(SphinxTranslator):
 
     def visit_productionlist(self, node: Element) -> None:
         self.new_state()
-        names = []
         productionlist = cast(Iterable[addnodes.production], node)
-        for production in productionlist:
-            names.append(production['tokenname'])
+        names = (production['tokenname'] for production in productionlist)
         maxlen = max(len(name) for name in names)
         lastname = None
         for production in productionlist:
@@ -890,7 +884,8 @@ class TextTranslator(SphinxTranslator):
 
     def visit_table(self, node: Element) -> None:
         if hasattr(self, 'table'):
-            raise NotImplementedError('Nested tables are not supported.')
+            msg = 'Nested tables are not supported.'
+            raise NotImplementedError(msg)
         self.new_state(0)
         self.table = Table()
 
@@ -1284,7 +1279,7 @@ class TextTranslator(SphinxTranslator):
         if 'text' in node.get('format', '').split():
             self.new_state(0)
             self.add_text(node.astext())
-            self.end_state(wrap = False)
+            self.end_state(wrap=False)
         raise nodes.SkipNode
 
     def visit_math(self, node: Element) -> None:
