@@ -2,11 +2,12 @@
 
 from __future__ import annotations
 
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from docutils.writers.docutils_xml import Writer as BaseXMLWriter
 
-from sphinx.builders import Builder
+if TYPE_CHECKING:
+    from sphinx.builders import Builder
 
 
 class XMLWriter(BaseXMLWriter):
@@ -47,5 +48,5 @@ class PseudoXMLWriter(BaseXMLWriter):
         self.output = self.document.pformat()
 
     def supports(self, format: str) -> bool:
-        """This writer supports all format-specific elements."""
+        """All format-specific elements are supported."""
         return True
